package kerho;

import java.io.File;
import java.util.Collection;
import java.util.List;

/**
 * Kerho-luokka, joka huolehtii jäsenistöstä.  Pääosin kaikki metodit
 * ovat vain "välittäjämetodeja" jäsenistöön.
 *
 * @author Vesa Lappalainen
 * @version 1.0, 09.02.2003
 * @version 1.1, 23.02.2003
 * @version 1.2, 07.01.2008 / testit
 * @version 1.3, 03.03.2013 / Harrastukset
 * 
 * Testien alustus
 * @example
 * <pre name="testJAVA">
 * #import kerho.SailoException;
 *  private Kerho kerho;
 *  private Jasen aku1;
 *  private Jasen aku2;
 *  private int jid1;
 *  private int jid2;
 *  private Harrastus pitsi21;
 *  private Harrastus pitsi11;
 *  private Harrastus pitsi22; 
 *  private Harrastus pitsi12; 
 *  private Harrastus pitsi23;
 *  
 *  @SuppressWarnings("javadoc")
 *  public void alustaKerho() {
 *    kerho = new Kerho();
 *    aku1 = new Jasen(); aku1.vastaaAkuAnkka(); aku1.rekisteroi();
 *    aku2 = new Jasen(); aku2.vastaaAkuAnkka(); aku2.rekisteroi();
 *    jid1 = aku1.getTunnusNro();
 *    jid2 = aku2.getTunnusNro();
 *    pitsi21 = new Harrastus(jid2); pitsi21.vastaaPitsinNyplays(jid2);
 *    pitsi11 = new Harrastus(jid1); pitsi11.vastaaPitsinNyplays(jid1);
 *    pitsi22 = new Harrastus(jid2); pitsi22.vastaaPitsinNyplays(jid2); 
 *    pitsi12 = new Harrastus(jid1); pitsi12.vastaaPitsinNyplays(jid1); 
 *    pitsi23 = new Harrastus(jid2); pitsi23.vastaaPitsinNyplays(jid2);
 *    try {
 *    kerho.lisaa(aku1);
 *    kerho.lisaa(aku2);
 *    kerho.lisaa(pitsi21);
 *    kerho.lisaa(pitsi11);
 *    kerho.lisaa(pitsi22);
 *    kerho.lisaa(pitsi12);
 *    kerho.lisaa(pitsi23);
 *    } catch ( Exception e) {
 *       System.err.println(e.getMessage());
 *    }
 *  }
 * </pre>
*/
public class Kerho  {
    private Jasenet jasenet = new Jasenet();
    private Harrastukset harrastukset = new Harrastukset();


    /**
     * Poistaa jäsenistöstä ja harrasteista ne joilla on nro. Kesken.
     * @param nro viitenumero, jonka mukaan poistetaan
     * @return montako jäsentä poistettiin
     */
    public int poista(@SuppressWarnings("unused") int nro) {
        return 0;
    }


    /**
     * Lisää kerhoon uuden jäsenen
     * @param jasen lisättävä jäsen
     * @throws SailoException jos lisäystä ei voida tehdä
     * @example
     * <pre name="test">
     * #THROWS SailoException  
     *  alustaKerho();
     *  kerho.etsi("*",0).size() === 2;
     *  kerho.lisaa(aku1);
     *  kerho.etsi("*",0).size() === 3;
     */
    public void lisaa(Jasen jasen) throws SailoException {
        jasenet.lisaa(jasen);
    }


    /** 
     * Korvaa jäsenen tietorakenteessa.  Ottaa jäsenen omistukseensa. 
     * Etsitään samalla tunnusnumerolla oleva jäsen.  Jos ei löydy, 
     * niin lisätään uutena jäsenenä. 
     * @param jasen lisätäävän jäsenen viite.  Huom tietorakenne muuttuu omistajaksi 
     * @throws SailoException jos tietorakenne on jo täynnä 
     * @example
     * <pre name="test">
     * #THROWS SailoException  
     *  alustaKerho();
     *  kerho.etsi("*",0).size() === 2;
     *  kerho.korvaaTaiLisaa(aku1);
     *  kerho.etsi("*",0).size() === 2;
     * </pre>
     */ 
    public void korvaaTaiLisaa(Jasen jasen) throws SailoException { 
        jasenet.korvaaTaiLisaa(jasen); 
    } 

    
    /** 
     * Korvaa harrastuksen tietorakenteessa.  Ottaa harrastuksen omistukseensa. 
     * Etsitään samalla tunnusnumerolla oleva harrastus.  Jos ei löydy, 
     * niin lisätään uutena harrastuksena. 
     * @param harrastus lisärtävän harrastuksen viite.  Huom tietorakenne muuttuu omistajaksi 
     * @throws SailoException jos tietorakenne on jo täynnä 
     */ 
    public void korvaaTaiLisaa(Harrastus harrastus) throws SailoException { 
        harrastukset.korvaaTaiLisaa(harrastus); 
    } 

    
    /**
     * Listään uusi harrastus kerhoon
     * @param har lisättävä harrastus 
     * @throws SailoException jos tulee ongelmia
     */
    public void lisaa(Harrastus har) throws SailoException {
        harrastukset.lisaa(har);
    }


    /** 
     * Palauttaa "taulukossa" hakuehtoon vastaavien jäsenten viitteet 
     * @param hakuehto hakuehto  
     * @param k etsittävän kentän indeksi  
     * @return tietorakenteen löytyneistä jäsenistä 
     * @throws SailoException Jos jotakin menee väärin
     * @example 
     * <pre name="test">
     *   #THROWS CloneNotSupportedException, SailoException
     *   alustaKerho();
     *   // TODO: tee testit kun etsi on korjattu
     * </pre>
     */ 
    public Collection<Jasen> etsi(String hakuehto, int k) throws SailoException { 
        return jasenet.etsi(hakuehto, k); 
    } 
    
    
    /**
     * Haetaan kaikki jäsen harrastukset
     * @param jasen jäsen jolle harrastuksia haetaan
     * @return tietorakenne jossa viiteet löydetteyihin harrastuksiin
     * @throws SailoException jos tulee ongelmia
     * @example
     * <pre name="test">
     * #THROWS SailoException
     * #import java.util.*;
     * 
     *  alustaKerho();
     *  Jasen aku3 = new Jasen();
     *  aku3.rekisteroi();
     *  kerho.lisaa(aku3);
     *  
     *  List<Harrastus> loytyneet;
     *  loytyneet = kerho.annaHarrastukset(aku3);
     *  loytyneet.size() === 0; 
     *  loytyneet = kerho.annaHarrastukset(aku1);
     *  loytyneet.size() === 2; 
     *  loytyneet.get(0) == pitsi11 === true;
     *  loytyneet.get(1) == pitsi12 === true;
     *  loytyneet = kerho.annaHarrastukset(aku2);
     *  loytyneet.size() === 3; 
     *  loytyneet.get(0) == pitsi21 === true;
     * </pre> 
     */
    public List<Harrastus> annaHarrastukset(Jasen jasen) throws SailoException {
        return harrastukset.annaHarrastukset(jasen.getTunnusNro());
    }


    /**
     * Asettaa tiedostojen perusnimet
     * @param nimi uusi nimi
     */
    public void setTiedosto(String nimi) {
        File dir = new File(nimi);
        dir.mkdirs();
        String hakemistonNimi = "";
        if ( !nimi.isEmpty() ) hakemistonNimi = nimi +"/";
        jasenet.setTiedostonPerusNimi(hakemistonNimi + "nimet");
        harrastukset.setTiedostonPerusNimi(hakemistonNimi + "harrastukset");
    }
    
      
    /**
     * Lukee kerhon tiedot tiedostosta
     * @param nimi jota käyteään lukemisessa
     * @throws SailoException jos lukeminen epäonnistuu
     * 
     * @example
     * <pre name="test">
     * #THROWS SailoException 
     * #import java.io.*;
     * #import java.util.*;
     *   
     *  String hakemisto = "testikelmit";
     *  File dir = new File(hakemisto);
     *  File ftied  = new File(hakemisto+"/nimet.dat");
     *  File fhtied = new File(hakemisto+"/harrastukset.dat");
     *  dir.mkdir();  
     *  ftied.delete();
     *  fhtied.delete();
     *  kerho = new Kerho(); // tiedostoja ei ole, tulee poikkeus 
     *  kerho.lueTiedostosta(hakemisto); #THROWS SailoException
     *  alustaKerho();
     *  kerho.setTiedosto(hakemisto); // nimi annettava koska uusi poisti sen
     *  kerho.tallenna(); 
     *  kerho = new Kerho();
     *  kerho.lueTiedostosta(hakemisto);
     *  Collection<Jasen> kaikki = kerho.etsi("",-1); 
     *  Iterator<Jasen> it = kaikki.iterator();
     *  it.next() === aku1;
     *  it.next() === aku2;
     *  it.hasNext() === false;
     *  List<Harrastus> loytyneet = kerho.annaHarrastukset(aku1);
     *  Iterator<Harrastus> ih = loytyneet.iterator();
     *  ih.next() === pitsi11;
     *  ih.next() === pitsi12;
     *  ih.hasNext() === false;
     *  loytyneet = kerho.annaHarrastukset(aku2);
     *  ih = loytyneet.iterator();
     *  ih.next() === pitsi21;
     *  ih.next() === pitsi22;
     *  ih.next() === pitsi23;
     *  ih.hasNext() === false;
     *  kerho.lisaa(aku2);
     *  kerho.lisaa(pitsi23);
     *  kerho.tallenna(); // tekee molemmista .bak
     *  ftied.delete()  === true;
     *  fhtied.delete() === true;
     *  File fbak = new File(hakemisto+"/nimet.bak");
     *  File fhbak = new File(hakemisto+"/harrastukset.bak");
     *  fbak.delete() === true;
     *  fhbak.delete() === true;
     *  dir.delete() === true;
     * </pre>
     */
    public void lueTiedostosta(String nimi) throws SailoException {
        jasenet = new Jasenet(); // jos luetaan olemassa olevaan niin helpoin tyhjentää näin
        harrastukset = new Harrastukset();

        setTiedosto(nimi);
        jasenet.lueTiedostosta();
        harrastukset.lueTiedostosta();
    }


    /**
     * Tallenttaa kerhon tiedot tiedostoon.  
     * Vaikka jäsenten tallettamien epäonistuisi, niin yritetään silti tallettaa
     * harrastuksia ennen poikkeuksen heittämistä.
     * @throws SailoException jos tallettamisessa ongelmia
     */
    public void tallenna() throws SailoException {
        String virhe = "";
        try {
            jasenet.tallenna();
        } catch ( SailoException ex ) {
            virhe = ex.getMessage();
        }

        try {
            harrastukset.tallenna();
        } catch ( SailoException ex ) {
            virhe += ex.getMessage();
        }
        if ( !"".equals(virhe) ) throw new SailoException(virhe);
    }


    /**
     * Testiohjelma kerhosta
     * @param args ei käytössä
     */
    public static void main(String args[]) {
        Kerho kerho = new Kerho();

        try {
            // kerho.lueTiedostosta("kelmit");

            Jasen aku1 = new Jasen(), aku2 = new Jasen();
            aku1.rekisteroi();
            aku1.vastaaAkuAnkka();
            aku2.rekisteroi();
            aku2.vastaaAkuAnkka();

            kerho.lisaa(aku1);
            kerho.lisaa(aku2);
            int id1 = aku1.getTunnusNro();
            int id2 = aku2.getTunnusNro();
            Harrastus pitsi11 = new Harrastus(id1);
            pitsi11.vastaaPitsinNyplays(id1);
            kerho.lisaa(pitsi11);
            Harrastus pitsi12 = new Harrastus(id1);
            pitsi12.vastaaPitsinNyplays(id1);
            kerho.lisaa(pitsi12);
            Harrastus pitsi21 = new Harrastus(id2);
            pitsi21.vastaaPitsinNyplays(id2);
            kerho.lisaa(pitsi21);
            Harrastus pitsi22 = new Harrastus(id2);
            pitsi22.vastaaPitsinNyplays(id2);
            kerho.lisaa(pitsi22);
            Harrastus pitsi23 = new Harrastus(id2);
            pitsi23.vastaaPitsinNyplays(id2);
            kerho.lisaa(pitsi23);

            System.out.println("============= Kerhon testi =================");
            Collection<Jasen> jasenet = kerho.etsi("", -1);
            int i = 0;
            for (Jasen jasen: jasenet) {
                System.out.println("Jäsen paikassa: " + i);
                jasen.tulosta(System.out);
                List<Harrastus> loytyneet = kerho.annaHarrastukset(jasen);
                for (Harrastus harrastus : loytyneet)
                    harrastus.tulosta(System.out);
                i++;
            }

        } catch ( SailoException ex ) {
            System.out.println(ex.getMessage());
        }
    }

}
